package uebung3;

import javafx.geometry.*;
import javafx.scene.*;
import javafx.scene.effect.*;
import javafx.scene.image.*;
import javafx.scene.layout.*;
import javafx.scene.paint.*;
import javafx.scene.text.*;
import javafx.stage.*;

import java.awt.image.*;
import java.io.*;
import java.lang.*;
import javax.imageio.*;

function arrange(number, aspect) {
    var w:Double = 1;
    var h:Double = 1;
    while(w*h<number) {
        if(w/h<aspect) w++ else h++
    }
    return [w as Integer,h as Integer];
}

function run(__ARGS__ : String[]) {
    def directory: File = new File(if(sizeof __ARGS__>0) __ARGS__[0] else ".");
    var selected: String = "";

    var scene: Scene = Scene {
        fill: LinearGradient {
            startX: 0, startY: 0
            endX: 0, endY: 1
            proportional: true
            stops: [
                Stop { offset: 0.0 color: Color.rgb(128,128,128) },
                Stop { offset: 1.0 color: Color.rgb(32,32,32) },
            ]
        }
        content: bind if(sizeof images[image | image.id==selected]>0) [content,selection] else { if(sizeof content.content>0) content else text }
    }

    var images: ImageView[];

    var content: Tile = Tile {
        width: bind scene.width
        height: bind scene.height
        nodeHPos: HPos.CENTER
        nodeVPos: VPos.CENTER
        hpos: HPos.CENTER
        vpos: VPos.CENTER
        hgap: 10, vgap: 10
        tileWidth: bind Math.floor(scene.width/content.columns-content.hgap)
        tileHeight: bind Math.floor(scene.height/content.rows-content.vgap)
        columns: bind arrange(sizeof images,scene.width/(scene.height as Double))[0]
        //rows: bind arrange(sizeof images,scene.width/(scene.height as Double))[1]
        content: bind images
    }

    var selection: VBox = VBox {
        width: bind scene.width
        height: bind scene.height
        nodeHPos: HPos.CENTER
        hpos: HPos.CENTER
        vpos: VPos.CENTER
        spacing: 10
        content: [
            ImageView {
                blocksMouse: true
                image: bind images[image | image.id==selected][0].image
                fitWidth: bind scene.width*0.9
                fitHeight: bind scene.height*0.9
                preserveRatio: true
                smooth: true
                cache: true
                effect: DropShadow { offsetX: 2, offsetY: 2 }
                onMouseClicked: function(self) { selected = "" }
            },
            Text {
                font: Font.font("Verdana",FontWeight.BOLD,12)
                fill: Color.WHITE
                textAlignment: TextAlignment.CENTER
                content: bind "{selected}"
                effect: DropShadow { radius: 2, spread: 0.5, offsetX: 0, offsetY: 0 }
            }
        ]
    }

    var text: VBox = VBox {
        width: bind scene.width
        height: bind scene.height
        nodeHPos: HPos.CENTER
        hpos: HPos.CENTER
        vpos: VPos.CENTER
        content: [
            Text {
                font: Font.font("Verdana",FontWeight.BOLD,12)
                fill: Color.WHITE
                textAlignment: TextAlignment.CENTER
                content: "Der Ordner enthält keine Bilder.\n(\"{directory.getCanonicalPath()}\")"
                effect: DropShadow { radius: 2, spread: 0.5, offsetX: 0, offsetY: 0 }
            }
        ]
    }

    FileMonitor {
        file: directory
        listener: FileMonitorListener {
            override function fileAdded(file: File) : Void {
                try {
                    var img:BufferedImage = ImageIO.read(file);
                    if(img!=null) {
                        println("Added {file.getName()}");
                        images = [images, ImageView {
                            id: file.getName()
                            image: javafx.ext.swing.SwingUtils.toFXImage(img)
                            fitWidth: bind content.tileWidth
                            fitHeight: bind content.tileHeight
                            preserveRatio: true
                            smooth: true
                            cache: true
                            effect: DropShadow { offsetX: 2, offsetY: 2 }
                            onMouseClicked: function(self) { selected = self.source.id }
                        }]
                    }
                } catch (e:Exception) {}
            }
            override function fileChanged(file: File) : Void {
                try {
                    var img:BufferedImage = ImageIO.read(file);
                    if(img!=null) {
                        println("Changed {file.getName()}");
                        var image:ImageView = scene.lookup(file.getName()) as ImageView;
                        image.image = javafx.ext.swing.SwingUtils.toFXImage(img)
                    }
                } catch (e:Exception) {}
            }
            override function fileRemoved(file: File) : Void {
                try {
                    println("Removed {file.getName()}");
                    images = images[image | image.id!=file.getName()]
                } catch (e:Exception) {}
            }
        }
    }.start();

    Stage {
        title: "Blatt03"
        width: 640, height: 480
        resizable: true
        scene: scene
    }
}